"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const nodepath = require("path");
const mfs = require("m-fs");
const titleExtractor_1 = require("./titleExtractor");
const MarkdownGenerator = require("./markdownGenerator");
const eventArgs_1 = require("./eventArgs");
const rename = require('node-rename-path');
const escapeHTML = require('escape-html');
const ROOTPROJ = 'root.json';
const DIR_PATHBAR_HTML = '__dir.path.g.html';
const DIR_CONTENT_HTML = '__dir.content.g.html';
const DIR_TITLE_HTML = '__dir.t.g.txt';
const FILE_TITLE_EXT = '.t.g.txt';
const dirCache = {};
class Processor {
    constructor(config, generator) {
        this.config = config;
        this.generator = generator;
        this.ignoredFiles = {};
        const ignoredFiles = [titleExtractor_1.default.TITLE_FILE, '.DS_Store', 'thumbs.db'];
        for (const file of ignoredFiles) {
            this.ignoredFiles[file] = true;
        }
    }
    startFromFile(relFile) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('process-file', {
                relFile,
            });
            yield this.markdownToHTML(relFile);
            const relDir = nodepath.dirname(relFile);
            this.logger.error('lili', {
                relDir,
            });
            const components = yield this.processDir(relDir, 1);
            if (!components.length) {
                throw new Error('Internal error: components should not be empty');
            }
            const dirComponent = components[0];
            const title = yield titleExtractor_1.default.fromFile(this.makeSrcPath(relFile));
            yield this.writeTitleFileForFile(relFile, title, dirComponent.tryGetAttachedName());
        });
    }
    processDir(relDir, stackCount) {
        return __awaiter(this, void 0, void 0, function* () {
            if (stackCount >= 100) {
                throw new Error('Potential infinite loop detected.');
            }
            this.logger.info('process-dir.started', {
                relDir,
            });
            if (dirCache[relDir]) {
                this.logger.info('process-dir.found-in-cache', {
                    relDir,
                });
                return dirCache[relDir];
            }
            this.logger.info('process-dir.NOT-found-in-cache', {
                relDir,
            });
            const absDir = this.makeSrcPath(relDir);
            const curComponent = yield this.pathComponentFromDir(absDir);
            const reversedComponents = [curComponent];
            if (relDir !== '.' && (yield this.isRootDir(absDir)) === false) {
                this.logger.info('process-dir.NOT-root', {
                    relDir,
                });
                const parentRelDir = nodepath.dirname(relDir);
                const parents = yield this.processDir(parentRelDir, stackCount + 1);
                if (!parents.length) {
                    throw new Error(`${relDir} is not a root dir, but got an empty PathBarItem array from parents.`);
                }
                const prevComponent = parents[0];
                curComponent.updateParentURL(parents[0].fullURL);
                if (!curComponent.attachedDisplayName) {
                    curComponent.attachedDisplayName = prevComponent.attachedDisplayName;
                    curComponent.attachedDisplayNameHTML = prevComponent.attachedDisplayNameHTML;
                }
                for (const p of parents) {
                    reversedComponents.push(p);
                }
            }
            const destPathBarHtml = nodepath.join(this.makeDestPath(relDir), DIR_PATHBAR_HTML);
            const pathBarHtml = this.generator.generatePathBarHtml(reversedComponents.slice().reverse());
            this.logger.info('process-dir.write-pathbar', {
                relDir, destPathBarHtml,
            });
            yield mfs.writeFileAsync(destPathBarHtml, pathBarHtml);
            this.logger.info('process-dir.leaf-dir-checking.started', {
                absDir,
            });
            const isLeafDir = yield this.isLeafDir(absDir);
            let childComponents;
            this.logger.info('process-dir.leaf-dir-checking.ended', {
                absDir, isLeafDir,
            });
            if (isLeafDir) {
                this.logger.info('process-dir.list-subfiles.started', {
                    absDir,
                });
                let subfiles = yield mfs.listSubFiles(absDir);
                subfiles = subfiles.filter((file) => !this.ignoredFiles[file]);
                if (!subfiles.length) {
                    throw new Error(`No files found in "${absDir}"`);
                }
                childComponents = yield this.childComponentsFromFiles(subfiles);
            }
            else {
                this.logger.info('process-dir.list-subdirs.started', {
                    absDir,
                });
                const subdirs = yield mfs.listSubDirs(absDir);
                childComponents = yield this.childComponentsFromDirs(subdirs);
                if (!subdirs.length) {
                    throw new Error(`No dirs found in "${absDir}"`);
                }
            }
            const contentHtml = this.generator.generateContentHtml(childComponents);
            const contentPath = nodepath.join(this.makeDestPath(relDir), DIR_CONTENT_HTML);
            this.logger.info('process-dir.write-contentHtml', {
                relDir, contentPath,
            });
            yield mfs.writeFileAsync(contentPath, contentHtml);
            const title = yield titleExtractor_1.default.fromDir(absDir);
            yield this.writeTitleFileForDir(relDir, title, curComponent.tryGetAttachedName());
            dirCache[relDir] = reversedComponents;
            return reversedComponents;
        });
    }
    isRootDir(dir) {
        return __awaiter(this, void 0, void 0, function* () {
            const rootFile = nodepath.join(dir, ROOTPROJ);
            return yield mfs.fileExists(rootFile);
        });
    }
    makeSrcPath(relFile) {
        return nodepath.join(this.config.srcDir, relFile);
    }
    makeDestPath(relFile) {
        return nodepath.join(this.config.destDir, relFile);
    }
    markdownToHTML(relFile) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('markdown2html-start', {
                relFile,
            });
            const content = yield mfs.readTextFileAsync(this.makeSrcPath(relFile));
            const html = MarkdownGenerator.convert(content);
            const htmlFile = MarkdownGenerator.rename(this.makeDestPath(relFile));
            this.logger.info('markdown2html-write', {
                htmlFile,
            });
            yield mfs.writeFileAsync(htmlFile, html);
        });
    }
    pathComponentFromDir(dir) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.verbose('pathComponentFromDir', {
                dir,
            });
            const dirName = nodepath.basename(dir);
            const title = yield titleExtractor_1.default.fromDir(dir);
            const component = new eventArgs_1.PathComponent(dirName, title);
            const attachedTitle = yield titleExtractor_1.default.attachedTitleFromDir(dir);
            if (attachedTitle) {
                this.logger.verbose('pathComponentFromDir.found-attached-title', {
                    dir, attachedTitle,
                });
                component.sourceOfAttachedName = true;
                component.attachedDisplayName = attachedTitle;
                component.attachedDisplayNameHTML = escapeHTML(attachedTitle);
            }
            return component;
        });
    }
    pathComponentFromFile(file) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.verbose('pathComponentFromFile', {
                file,
            });
            const name = nodepath.parse(file).name;
            const title = yield titleExtractor_1.default.fromFile(file);
            return new eventArgs_1.PathComponent(name, title);
        });
    }
    childComponentsFromDirs(dirs) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield Promise.all(dirs.map((d) => __awaiter(this, void 0, void 0, function* () {
                return yield this.pathComponentFromDir(d);
            })));
        });
    }
    childComponentsFromFiles(files) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield Promise.all(files.map((file) => __awaiter(this, void 0, void 0, function* () {
                return yield this.pathComponentFromFile(file);
            })));
        });
    }
    isLeafDir(absDir) {
        return __awaiter(this, void 0, void 0, function* () {
            const subdirs = yield mfs.listSubDirs(absDir);
            return subdirs.length === 0;
        });
    }
    writeTitleFileForFile(relFile, title, attached) {
        return __awaiter(this, void 0, void 0, function* () {
            const html = this.tryEscapeTitle(title + (attached || ''));
            const gRelFile = rename(relFile, (pathObj) => {
                pathObj.ext = FILE_TITLE_EXT;
            });
            yield mfs.writeFileAsync(this.makeDestPath(gRelFile), html);
        });
    }
    writeTitleFileForDir(relDir, title, attached) {
        return __awaiter(this, void 0, void 0, function* () {
            const html = this.tryEscapeTitle(title + (attached || ''));
            const dest = nodepath.join(this.makeDestPath(relDir), DIR_TITLE_HTML);
            yield mfs.writeFileAsync(dest, html);
        });
    }
    tryEscapeTitle(title) {
        if (this.config.escapeTitle) {
            return escapeHTML(title);
        }
        return title;
    }
    get logger() {
        return this.config.logger;
    }
}
exports.Processor = Processor;
//# sourceMappingURL=core.js.map